%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% synthetic_mcdn:
%
% Generates a latent variable SEM following a simple pattern: every latent
% variable has the same number of indicators, and each indicator has a
% single parent. On top of it, bi-directed structure among indicators is
% added, but pruned so that the model is identifiable.
%
% This creates a model based on the binary MCDN formulation.
%
% Created by: Ricardo Silva, London
% University College London
%
% Current version: 10/10/2011

function [Y Y_parents bidir_graph L_true copula_theta copula_true ...
          expo_y true_cov_X] = ...
    synthetic_mcdn(num_x, num_ind, num_y, prob_edge, signal_range, ...
                   marg_range, cop_range, N)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Generate core structure

dir_graph = zeros(num_y, num_x); % Directed graph
for x = 1:num_x
  s_y = num_ind * (x - 1) + 1; e_y = num_ind * x;
  dir_graph(s_y:e_y, x) = 1;
end

bidir_graph = rand(num_y) < prob_edge;
for y1 = 1:num_y
  bidir_graph(y1, y1) = 0;
  for y2 = (y1 + 1):num_y
    bidir_graph(y2, y1) = bidir_graph(y1, y2);
  end
end

X_children = cell(num_x, 1);
Y_parents = zeros(num_y, 1);

for x = 1:num_x
  X_children{x} = find(dir_graph(:, x) == 1);
  Y_parents(X_children{x}) = x;
end

  
% "Simplify" it to make model identifiable
for x = 1:num_x
  found = false;
  n_x = length(X_children{x});
  while ~found
    for y1_idx = 1:n_x
      for y2_idx = (y1_idx + 1):n_x
        for y3_idx = (y2_idx + 1):n_x
           y1 = X_children{x}(y1_idx); 
           y2 = X_children{x}(y2_idx); 
           y3 = X_children{x}(y3_idx);
           if sum(sum(bidir_graph([y1 y2 y3], [y1 y2 y3]))) == 0
             found = true;
             break
           end
           if found, break; end
        end
        if found, break; end
      end
    end
    if ~found
      num_edges = sum(sum(bidir_graph(X_children{x}, X_children{x}))) / 2;
      pick_e = randsample(num_edges, 1);
      e_idx = 0;
      for y1_idx = 1:n_x
        for y2_idx = (y1_idx + 1):n_x
          y1 = X_children{x}(y1_idx); 
          y2 = X_children{x}(y2_idx); 
          if bidir_graph(y1, y2) == 1
             e_idx = e_idx + 1; 
             if e_idx == pick_e
               bidir_graph(y1, y2) = 0;
               bidir_graph(y2, y1) = 0;               
               break
             end
          end          
        end
        if e_idx == pick_e, break; end
      end
    end
  end
end

%for x1 = 1:num_x
%  a1 = num_ind * (x1 - 1) + 1;
%  for x2 = (x1 + 1):num_x
%    a2 = num_ind * (x2 - 1) + 1;
%    bidir_graph(a1, a2) = 0; bidir_graph(a2, a1) = 0;
%    bidir_graph(a1, a2 + 1) = 0; bidir_graph(a2 + 1, a1) = 0;
%    bidir_graph(a1 + 1, a2 + 1) = 0; bidir_graph(a2 + 1, a1 + 1) = 0;
%  end
%  bidir_graph(a1, a1 + 1) = 0; bidir_graph(a1 + 1, a1) = 0;
%end

% Further simplify it to avoid models with very weak correlations, strictly
% forbid any graph where vertices might have more than 3 neighbors

num_neighbors = sum(bidir_graph);
while any(num_neighbors > 3)
  [~, elim_1] = max(num_neighbors);
  elim_2 = -1; max_e = -1;
  nn = find(bidir_graph(elim_1, :) == 1);
  for y = nn
     if num_neighbors(y) > max_e
       elim_2 = y;
       max_e = num_neighbors(y);
     end
  end
  bidir_graph(elim_1, elim_2) = 0;
  bidir_graph(elim_2, elim_1) = 0;
  num_neighbors = sum(bidir_graph);
end

fprintf('Bi-directed graph\n')
disp(bidir_graph)

fprintf('Number of spouses:\n')
disp(sum(bidir_graph))

num_edges = sum(sum(bidir_graph)) / 2;
edges = graph_edge_list(bidir_graph);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Model parameters

% Loading structure
L_true = zeros(num_y, 2);
for y = 1:num_y
  signal = rand() * (signal_range(2) - signal_range(1)) + signal_range(1);
  sign = (rand() > 0.5) * 2 - 1;
  L_true(y, 1) = sign * sqrt(signal / (1 - signal));
  marg = rand() * (marg_range(2) - marg_range(1)) + marg_range(1);
  L_true(y, 2) = -norminv(marg) * sqrt(L_true(y, 1)^2 + 1);
end

% Copula structure
copula_theta = rand(num_edges, 1) * (cop_range(2) - cop_range(1)) + cop_range(1);
copula_theta = abs(copula_theta); % Only positive values allowed here

copula_true = zeros(num_y);
for i = 1:num_edges
  copula_true(edges(i, 1), edges(i, 2)) = copula_theta(i);
  copula_true(edges(i, 2), edges(i, 1)) = copula_theta(i);
end

expo_y = get_expo_y(1:num_y, bidir_graph);

% Covariance of latents
L_X = randn(num_x); 
true_cov_X = corr(L_X' * L_X);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Data generation

X_data = mvnrnd(zeros(N, num_x), true_cov_X);
y_data = sample_archimedean(L_true, expo_y, copula_theta, Y_parents, ...
                             bidir_graph, X_data);  
Y = 2 * y_data - 1; 
