%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% fit_rho_X_fixed_exp:
%
% Optimize all latent correlations based on the composite likelihood score,
% by fixing the loadings and marginalizing the measurement correlation
% structure.
%
% THIS ASSUMES THERE IS ONLY ONE REGION IN 'REGIONS_Y' WHERE A PAIR OF
% INDICATORS OF DIFFERENT LATENT VARIABLES CAN APPEAR.
%
% Input:
%
% - L: loadings
% - Y_counts: pairwise sufficient statistics
% - Y_parents: list of parents for all observed variables
% - X_children: same information as Y_parents, but indexed by the latents
% - G: bi-directed structure
% - regions: a list of pairs of latent variables defining the regions
% - Y_exp: samples for the measurement exponents for each region
% - C: samples for the copula parameters for each region
% - is_w: sample weights
% - X_v: grid values shared by all latent variables
% - X_ws: cell structure containing latent weights for a range of possible 
%         correlations  
% - rho_domain: possible correlation values
%
% Output:
% 
% - score: the corresponding  composite log-likelihood scaled by N
% - gr: the corresponding gradient scaled by N
%
% Created by: Ricardo Silva, London, 05/03/2012
% University College London
% Current version: 08/03/2012

function rho_X = fit_rho_X_fixed_exp(L, Y_counts, ...
                           X_children, G, regions, expo_y, C, is_w, ...
                           X_v, X_ws, rho_domain, scale, ...
                           fixed_rho_X, fixed_rho_X_bool)

num_x = length(X_children);
num_quad = size(X_v, 1);
rho_X = eye(num_x);
for x1 = 1:num_x
  for x2 = (x1 + 1):num_x
    if fixed_rho_X_bool(x1, x2) == 1
      rho_X(x1, x2) = fixed_rho_X(x1, x2);
      rho_X(x2, x1) = rho_X(x1, x2);
    end
  end
end

% Calculate function

bin_values = [0 0; 0 1; 1 0; 1 1]'; signs = [1 -1 -1 1];

n_r = 1;

M1_ones = ones(1, num_quad);
num_rho = length(rho_domain);

for r = regions

  if fixed_rho_X_bool(r(1), r(2)), continue; end
  
  M = size(C{n_r}, 3); % Number of samples  
  M_ones = ones(M, num_quad);
  
  Y_scope = [X_children{r(1)}; X_children{r(2)}];
  Y_parents_r = [ones(length(X_children{r(1)}), 1); ...
                 2 * ones(length(X_children{r(2)}), 1)];  
  U = cdf_probit(L(Y_scope, :), Y_parents_r, X_v);
  scores = zeros(1, num_rho);
  num_1 = length(X_children{r(1)});
  
  for y_i_idx = 1:num_1
    
    y_i = Y_scope(y_i_idx);
        
    for y_j_idx = (num_1 + 1):length(Y_scope)
        
      y_j = Y_scope(y_j_idx);      
      if G(y_i, y_j) == 1          
        rep_u_i = repmat(U(:, y_i_idx)', M, 1);
        rep_u_j = repmat(U(:, y_j_idx)', M, 1);
        exp_i = expo_y(y_i, y_j);
        exp_j = expo_y(y_j, y_i);
        rho = cop_transform(squeeze(C{n_r}(y_i_idx, y_j_idx, :)), scale);
        rep_rho = repmat(rho, 1, num_quad);
        is_w_choice = squeeze(is_w{n_r}(y_i_idx, y_j_idx, :));
        M_iter = M;
      else
        rep_u_i = U(:, y_i_idx)';
        rep_u_j = U(:, y_j_idx)';
        is_w_choice = 1;
        M_iter = 1;
      end
           
      for v = bin_values % Iterate over all instantiations of {Y_i, Y_j}
                                
        D = zeros(M_iter, num_quad); 
        z_pos = 1;
        
        for z = bin_values % CDF to PMF transformation
            
          i_zv = v(1) - z(1); j_zv = v(2) - z(2); 
          if i_zv < 0 || j_zv < 0, continue; end                                   

          % SCORE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%          

          if G(y_i, y_j) == 1
            if i_zv == 0                
              u_i   = rep_u_i.^exp_i;
              u_i0  = rep_u_i.^(1 - exp_i);  
            else
              u_i   = M_ones;
              u_i0  = M_ones;
            end
            if j_zv == 0                
              u_j   = rep_u_j.^exp_j;
              u_j0  = rep_u_j.^(1 - exp_j);
            else
              u_j   = M_ones;
              u_j0  = M_ones;
            end
            aux = 1 + (exp(-rep_rho .* u_i) - 1) .* (exp(-rep_rho .* u_j) - 1) ./ ...
                      (exp(-rep_rho) - 1); 
            D = D + signs(z_pos) * (u_i0 .* u_j0 .* (-log(aux) ./ rep_rho));
          else 
            if i_zv == 0
              u_i0 = rep_u_i;
            else
              u_i0 = M1_ones;
            end  
            if j_zv == 0
              u_j0 = rep_u_j;
            else
              u_j0 = M1_ones;
            end
            D = D + signs(z_pos) * (u_i0 .* u_j0);
          end            
          
          % ITERATE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          
          z_pos = z_pos + 1;
          
        end % end of PMF transformation          
        
        pmfs = D * X_ws;
        N_ij = Y_counts{v(1) + 1, v(2) + 1}(y_i, y_j);
        scores = scores + N_ij * (is_w_choice' * log(pmfs));
        
      end % end of {Y_i, Y_j} calculations
      
      [~, best_rho_idx] = max(scores); 
      best_rho = rho_domain(best_rho_idx);
      fprintf('Best rho for [%d %d] is %f\n', y_i, y_j, best_rho);
      rho_X(r(1), r(2)) = best_rho; rho_X(r(2), r(1)) = best_rho;

    end % End of region optimization
    
  end

  fprintf('Best rho for [X%d -- X%d] is %f\n', r(1), r(2), best_rho);
  n_r = n_r + 1;
  
end
