%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% score_embedding:
%
% Score estimate of latent variable vector X
%
% Input:
%
% - X: latent values. A row vector
% - Y: the corresponding observation. *****ENCODED AS 0/1*****
% - L: loadings
% - rho_X: fixed latent correlations
% - Y_parents: the respective parents ofe ach observed variable
% - X_children: same information as Y_parents, but indexed by the latents
% - G: bi-directed structure
% - regions: a list of pairs of latent variables defining the regions
% - expo_y: exponents in each factor
% - cop_hat: estimate of copula parameters for each region
% - scale: parameter of transformation for copula parameters
%   
% Output:
% 
% - score: the corresponding  composite log-likelihood 
%
% Created by: Ricardo Silva, London, 27/03/2012
% University College London
% Current version: 27/03/2012

function [score score_gr] = score_embedding(X, Y, L, rho_X, ...
                 Y_parents, X_children, G, regions, expo_y, cop_hat)

num_x = size(rho_X, 1);

score = 0; 
score_gr = zeros(1, num_x);

% Calculate function

bin_values = [0 0; 0 1; 1 0; 1 1]'; signs = [1 -1 -1 1];

n_r = 1;

region_weight = [1 / (num_x - 1) 1; 1 1 / (num_x - 1)];

U = cdf_probit(L, Y_parents, X);
U_g = pdf_probit(L, Y_parents, X);

for r = regions;

  Y_scope = [X_children{r(1)}; X_children{r(2)}];
  num_y_r = length(Y_scope);  
    
  for y_i_idx = 1:num_y_r
    
    y_i = Y_scope(y_i_idx);
    x_i = Y_parents(y_i);
    U_i = U(y_i);
    U_i_g = U_g(y_i);
    
    if y_i_idx <= length(X_children{r(1)})
      p1 = 1;
    else
      p1 = 2;
    end
    
    for y_j_idx = (y_i_idx + 1):num_y_r
        
      y_j = Y_scope(y_j_idx);      
      x_j = Y_parents(y_j);
      U_j = U(y_j);
      U_j_g = U_g(y_j);
      if G(y_i, y_j) == 1          
        exp_i = expo_y(y_i, y_j);
        exp_j = expo_y(y_j, y_i);
        rho = cop_hat{n_r}(y_i_idx, y_j_idx);
      end
            
      if Y_parents(y_i) ~= Y_parents(y_j)
        if p1 == 1, p2 = 2; else p2 = 1; end
      else
        p2 = p1;
      end

      v = [Y(y_i); Y(y_j)];
                                
      pmf = 0; gr = [0 0];
      z_pos = 1;
        
      for z = bin_values % CDF to PMF transformation
            
        i_zv = v(1) - z(1); j_zv = v(2) - z(2); 
        if i_zv < 0 || j_zv < 0, continue; end                                   

        % SCORE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%          

        if G(y_i, y_j) == 1
          if i_zv == 0                
            u_i    = U_i^exp_i;
            u_i0   = U_i^(1 - exp_i);
          else
            u_i   = 1;
            u_i0  = 1;
          end
          if j_zv == 0                
            u_j    = U_j^exp_j;
            u_j0   = U_j^(1 - exp_j);
          else
            u_j   = 1;
            u_j0  = 1;
          end
          aux_C_1 = exp(-rho * u_i); 
          aux_C_2 = exp(-rho * u_j);
          aux_C_3 = exp(-rho);
          aux = 1 + (aux_C_1 - 1) * (aux_C_2 - 1) / (aux_C_3 - 1); 
          C_aux = -log(aux) / rho;
          pmf = pmf + signs(z_pos) * (u_i0 * u_j0) * C_aux;
        else 
          if i_zv == 0
            u_i0 = U_i;
          else
            u_i0 = 1;
          end  
          if j_zv == 0
            u_j0 = U_j;
          else
            u_j0 = 1;
          end
          pmf = pmf + signs(z_pos) * (u_i0 * u_j0);
        end            
          
        % GRADIENT %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%          

        if G(y_i, y_j) == 1
          if i_zv == 0 && j_zv == 0
            aux_C_4 = exp(-rho * (u_i + u_j));
            den = aux_C_3 + aux_C_4 - aux_C_1 - aux_C_2;
            C_aux_g_i = (aux_C_4 - aux_C_1) / den;
            C_aux_g_j = (aux_C_4 - aux_C_2) / den;
            gr(p1) = gr(p1) + signs(z_pos) * U_i_g * L(y_i, 1) * u_j0 * ...
                     ((1 - exp_i) * C_aux / u_i0 + exp_i * C_aux_g_i);
            gr(p2) = gr(p2) + signs(z_pos) * U_j_g * L(y_j, 1) * u_i0 * ...
                     ((1 - exp_j) * C_aux / u_j0 + exp_j * C_aux_g_j);
          elseif i_zv == 1 && j_zv == 0
            gr(p2) = gr(p2) + signs(z_pos) * U_j_g * L(y_j, 1);  
          elseif i_zv == 0 && j_zv == 1
            gr(p1) = gr(p1) + signs(z_pos) * U_i_g * L(y_i, 1);  
          end
        else            
          if i_zv == 0 && j_zv == 0
            gr(p1) = gr(p1) + signs(z_pos) * (U_i_g * U_j)   * L(y_i, 1);
            gr(p2) = gr(p2) + signs(z_pos) * (U_i   * U_j_g) * L(y_j, 1); 
          elseif i_zv == 1 && j_zv == 0            
            gr(p2) = gr(p2) + signs(z_pos) * U_j_g * L(y_j, 1); 
          elseif i_zv == 0 && j_zv == 1
            gr(p1) = gr(p1) + signs(z_pos) * U_i_g * L(y_i, 1);
          end
        end
        
        % ITERATE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
          
        z_pos = z_pos + 1;
          
      end % end of PMF transformation          
      
      if p1 ~= p2
        det_x = 1 - rho_X(x_i, x_j)^2;  
        log_pmf_x = -0.5 * log(det_x) ...
                    -0.5 * (X(x_i)^2 + X(x_j)^2 - 2 * rho_X(x_i, x_j) * X(x_i) * X(x_j)) / det_x;
        log_pmf_x_g = - [X(x_i) - rho_X(x_i, x_j) * X(x_j);...
                         X(x_j) - rho_X(x_i, x_j) * X(x_i)] / det_x;
      else
        log_pmf_x = -0.5 * X(x_i)^2;
        log_pmf_x_g = -X(x_i);
      end
      score = score + (log(pmf) + log_pmf_x) * region_weight(p1, p2);

      if p1 ~= p2
        score_gr(x_i) = score_gr(x_i) + ...
                        (gr(p1) / pmf + log_pmf_x_g(p1)) * region_weight(p1, p2);
        score_gr(x_j) = score_gr(x_j) + ...
                        (gr(p2) / pmf + log_pmf_x_g(p2)) * region_weight(p1, p2);                     
      else
        score_gr(x_i) = score_gr(x_i) + ...
                        (gr(p1) / pmf + log_pmf_x_g) * region_weight(p1, p2);
      end
      
    end
      
  end
  
  n_r = n_r + 1;
  
end

score = -score;
score_gr = -score_gr;
