%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% build_distr_approx:
%
% Generate a quick-and-dirty approximate representation of an univariate 
% distribution.
%
% Input:
%
% - M: number of points
% - theta: parameters
% - use_laplace: if true, build approximation for Laplace distribution
%
% Output:
% 
% - points: the points
% - w: the weights
%
% Created by: Ricardo Silva, London, 15/03/2012
% University College London
% Current version: 15/03/2012

function [points w] = build_distr_approx(M, theta, use_laplace)

if nargin == 3 && use_laplace
    
  error('build_cop_theta_prior_approx::laplace distribution not implemented yet');
  
else
    
  % theta is assumed to have three entries corresponding to the mean,
  % variance and cutoff of a truncated Gaussian
  
  mean_theta = theta(1); var_theta = theta(2); cutoff = theta(3);

  % Normalizing constant of a Gaussian (mean_theta, var_theta) truncated 
  % at both ends by '-cutoff' and 'cutoff'. It is assumed that 
  % mean_theta > -cutoff
  
  Z = normcdf(cutoff, mean_theta, var_theta) - ...
      normcdf(-cutoff, mean_theta, var_theta);
  
  % Now find the points where between which each bin has probability mass 
  % 1 / M
  
  lb = max(-cutoff, norminv(0.0001, mean_theta, var_theta));
  ub = min(cutoff,  norminv(0.9999, mean_theta, var_theta));
  points = zeros(M, 1);
  
  curr_point = lb;  
  
  for i = 1:(M - 1)
    point_acc = normcdf(curr_point, mean_theta, var_theta);  
    next_point = norminv(Z / M + point_acc, mean_theta, var_theta);
    points(i) = (next_point + curr_point) / 2;
    curr_point = next_point;
  end
  points(end) = (ub + curr_point) / 2;
  
  w = ones(M, 1) / M;
  
end
